@@ -59,7 +59,6 @@
 	[browser setAllowedExtensions:extensions];
 	[browser setBaseLocation:[preferences musicTabDir]];
 	
-	[njbTable setAllowedExtensions:extensions];
 	[njbTable setAutosaveName:PREF_KEY_MUSIC_TAB_TABLE];
 	[njbTable setAutosaveTableColumns:YES];
 }
@@ -67,7 +66,16 @@
 - (void)onConnectAndActive
 {
 	[super onConnectAndActive];
-	[self loadTracks];
+	if ([playlistsTab fullItemArray])
+	{
+		// playlistsTab has already got it
+		[fullItemArray release];
+		fullItemArray = [[MyMutableArray alloc] initWithMyMutableArray:[playlistsTab fullItemArray]];
+		[self clearSortingArrows];
+		[self searchNJBTable];
+	}
+	else
+		[self loadTracks];
 }
 
 - (SEL)sortSelectorForColumn:(NSString *)columnID
@@ -219,7 +227,6 @@
 {
 	if (![theNJB isConnected])
 		return;
-		
 	NJBQueueItem *copyTrack = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(downloadTrack:) withObject:track];
 	[copyTrack setStatus:STATUS_DOWNLOADING_TRACK];
 	NSString *description = [NSString stringWithFormat:@"Downloading track '%@'", [track title]];
@@ -235,11 +242,9 @@
 	if ([preferences writeTagAfterCopy] && [track codec] == CODEC_MP3)
 	{
 		NJBQueueItem *writeTag = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(writeTagToFile:) withObject:track];
-		// putting this to NO causes weird and wonderful crashes later on e.g. memory of the njb_t struct in theNJB getting overwritten...
-		// should be YES though (wasn't pre 0.23)
-		[writeTag setRunInMainThread:YES];
+		[writeTag setRunInMainThread:NO];
 		[writeTag setDisplayStatus:NO];
-
+		
 		[self addToQueue:copyTrack withSubItems:[NSArray arrayWithObjects:updatePlaylistsTab, writeTag, nil] withDescription:description];
 		[writeTag release];
 	}
@@ -251,14 +256,17 @@
 }
 
 /* copy the file at filename to
- * the NJB.
+ * the NJB
  */
 - (void)copyFileToNJB:(NSString *)filename
 {
-	Track *track = [[Track alloc] init];
+	ID3Tagger *tagger = [[ID3Tagger alloc] init];
+	[tagger setFileExtensionsToCodecsDictionary:fileExtensionsToCodecsDictionary];
+	[tagger setFilename:filename];
+	Track *track = [tagger readTrack];
 	[track setFullPath:filename];
 	[self copyTrackToNJB:track];
-	[track release];
+	[tagger release];
 }
 
 /* actually add the track to the queue
@@ -268,32 +276,30 @@
 {
 	if (![theNJB isConnected])
 		return;
-	NJBQueueItem *copyTrack = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(uploadTrack:) withObject:track];
+	NJBQueueItem *copyTrack = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(uploadTrack:) withObject:track];
 	[copyTrack setStatus:STATUS_UPLOADING_TRACK];
 	
-	NJBQueueItem *addToList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(addToFullItemArray:) withObject:track withRunInMainThread:YES];
-	[addToList setDisplayStatus:NO];
+	NJBQueueItem *addToTrackList = [[NJBQueueItem alloc] initWithTarget:fullItemArray withSelector:@selector(addObject:) withObject:track withRunInMainThread:YES];
+	[addToTrackList setDisplayStatus:NO];
 	
-	[copyTrack cancelItemIfFail:addToList];
+	// call searchNJBTable to pretend the search list changed to update
+	// saves data and does a reload
+	NJBQueueItem *updateTrackList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(searchNJBTable)];
+	[updateTrackList setRunInMainThread:YES];
+	[updateTrackList setDisplayStatus:NO];
 	
-	NSString *description = [NSString stringWithFormat:@"Uploading track '%@'", [track fullPath]];
-	[self addToQueue:copyTrack withSubItems:[NSArray arrayWithObjects:addToList, nil] withDescription:description];
+	// tell PlaylistsTab
+	NJBQueueItem *updatePlaylistsTab = [[NJBQueueItem alloc] initWithTarget:playlistsTab withSelector:@selector(updateTracks)];
+	[updatePlaylistsTab setRunInMainThread:NO];
+	[updatePlaylistsTab setDisplayStatus:NO];
+	
+	NSString *description = [NSString stringWithFormat:@"Uploading track '%@'", [track title]];
+	[self addToQueue:copyTrack withSubItems:[NSArray arrayWithObjects:addToTrackList, updateTrackList, updatePlaylistsTab, nil] withDescription:description];
 	
 	[copyTrack release];
-	[addToList release];
-}
-
-/* uploads the track to the jukebox, run in worker thread
- */
-- (NJBTransactionResult *)uploadTrack:(Track *)track
-{
-	ID3Tagger *tagger = [[ID3Tagger alloc] init];
-	[tagger setFileExtensionsToCodecsDictionary:fileExtensionsToCodecsDictionary];
-	[tagger setFilename:[track fullPath]];
-	Track *trackWithTag = [tagger readTrack];
-	[track setValuesToTrack:trackWithTag];
-	[tagger release];
-	return [theNJB uploadTrack:track];
+	[addToTrackList release];
+	[updateTrackList release];
+	[updatePlaylistsTab release];
 }
 
 /* write the info in track to the tag in the file
@@ -382,12 +388,11 @@
 			if (rowIndex == -1)
 				return;	
 			
-			Track *oldTrack = (Track *)[[self itemArrayDisplaying] objectAtIndex:rowIndex];
-			
-			NJBQueueItem *changeTrackTag = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(changeTrackTagTo:from:) withObject:modifiedTrack];
-			[changeTrackTag setObject2:oldTrack];
+			NJBQueueItem *changeTrackTag = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(changeTrackTag:) withObject:modifiedTrack];
 			[changeTrackTag setStatus:STATUS_UPDATING_TRACK_TAG];
 			
+			Track *oldTrack = (Track *)[[self itemArrayDisplaying] objectAtIndex:rowIndex];
+			
 			NJBQueueItem *updateTrackList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(replaceTrack:withTrack:)
 																																withObject:oldTrack withRunInMainThread:YES];
 			[updateTrackList setObject2:modifiedTrack];
@@ -399,15 +404,18 @@
 			[updateDisplay setRunInMainThread:YES];
 			[updateDisplay setDisplayStatus:NO];
 			
-			[updateTrackList cancelItemIfFail:updateTrackList];
-			[updateTrackList cancelItemIfFail:updateDisplay];
+			// tell PlaylistsTab
+			NJBQueueItem *updatePlaylistsTab = [[NJBQueueItem alloc] initWithTarget:playlistsTab withSelector:@selector(updateTracks)];
+			[updatePlaylistsTab setRunInMainThread:NO];
+			[updatePlaylistsTab setDisplayStatus:NO];
 			
 			NSString *description = [NSString stringWithFormat:@"Changing tag for track '%@'", [oldTrack title]];
-			[self addToQueue:changeTrackTag withSubItems:[NSArray arrayWithObjects:updateTrackList, updateDisplay, nil] withDescription:description];
+			[self addToQueue:changeTrackTag withSubItems:[NSArray arrayWithObjects:updateTrackList, updateDisplay, updatePlaylistsTab, nil] withDescription:description];
 
 			[changeTrackTag release];
 			[updateTrackList release];
 			[updateDisplay release];
+			[updatePlaylistsTab release];
 			
 			break;
 		case OBJECT_FILE_BROWSER:
@@ -428,15 +436,17 @@
 	// release old track list
 	[truncatedItemArray release];
 	
-	if ([searchString length] != 0)
+	if ([searchString length] == 0)
 	{
-		truncatedItemArray = [fullItemArray searchFor:searchString];
-		[truncatedItemArray retain];
-	}
-	else
 		truncatedItemArray = nil;
-	[njbTable reloadData];
+		[njbTable reloadData];
+		return;
+	}
 	
+	truncatedItemArray = [fullItemArray searchFor:searchString];
+	[truncatedItemArray retain];
+	
+	[njbTable reloadData];
 	if ([njbTable selectedRow] != -1)
 	{
 		// update display to show new track
@@ -490,8 +500,7 @@
 	}
 	else if ([aResponder class] == [NSMatrix class])
 		objectToSaveTrack = OBJECT_FILE_BROWSER;
-	// used to be [aResponder class] == [NSSearchField class]
-	else if (aResponder == njbSearchField)
+	else if ([aResponder class] == [NSSearchField class])
 	{
 		[drawerController disableAll];
 	}
@@ -516,15 +525,11 @@
 																							 withRunInMainThread:YES];
 	[updateTable setDisplayStatus:NO];
 	
-	[getTracks cancelItemIfFail:updateTable];
-	
 	NSString *description = @"Getting track list";
 	[self addToQueue:getTracks withSubItems:[NSArray arrayWithObjects:clearSortingArrows, updateTable, nil] withDescription:description];
 	[getTracks release];
 	[clearSortingArrows release];
 	[updateTable release];
-	// it's not actually yet, but will be in a while, don't want to go copying anything
-	trackListUpToDate = YES;
 }
 
 /* this is called from the queue consumer 
@@ -541,8 +546,6 @@
 	
 	[fullItemArray retain];
 	
-	trackListUpToDate = YES;
-	
 	return [result autorelease];
 }
 
@@ -551,7 +554,7 @@
 	Track *track = (Track *)item;
 
 	NJBQueueItem *deleteTrack = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(deleteTrack:)
-																												withObject:track];
+																												withObject:([NSNumber numberWithInt:[track trackID]])];
 	[deleteTrack setStatus:STATUS_DELETING_TRACK];
 		
 	NJBQueueItem *removeFromAllTracks = [[NJBQueueItem alloc] initWithTarget:fullItemArray withSelector:@selector(removeObject:)
@@ -565,15 +568,18 @@
 	[updateTrackList setRunInMainThread:YES];
 	[updateTrackList setDisplayStatus:NO];
 	
-	[deleteTrack cancelItemIfFail:removeFromAllTracks];
-	[deleteTrack cancelItemIfFail:updateTrackList];
-	
+	// tell PlaylistsTab
+	NJBQueueItem *updatePlaylistsTab = [[NJBQueueItem alloc] initWithTarget:playlistsTab withSelector:@selector(updateTracks)];
+	[updatePlaylistsTab setRunInMainThread:NO];
+	[updatePlaylistsTab setDisplayStatus:NO];
+		
 	NSString *description = [NSString stringWithFormat:@"Deleting track '%@'", [track title]];
-	[self addToQueue:deleteTrack withSubItems:[NSArray arrayWithObjects:removeFromAllTracks, updateTrackList, nil] withDescription:description];
-
+	[self addToQueue:deleteTrack withSubItems:[NSArray arrayWithObjects:removeFromAllTracks, updateTrackList, updatePlaylistsTab, nil] withDescription:description];
+		
 	[deleteTrack release];
 	[removeFromAllTracks release];
 	[updateTrackList release];
+	[updatePlaylistsTab release];
 		
 	NSLog(@"Deleting track id %d with title %@", [track trackID], [track title]);		
 }
@@ -594,15 +600,6 @@
 - (void)activate
 {
 	[super activate];
-	if (!trackListUpToDate)
-	{
-		[fullItemArray release];
-		fullItemArray = nil;
-		[truncatedItemArray release];
-		truncatedItemArray = nil;
-		[njbTable reloadData];
-		[self loadTracks];
-	}	
 	if (activeObject == OBJECT_NJB_TABLE)
 	{
 		if ([njbTable selectedRow] != -1)
@@ -631,12 +628,4 @@
 	[super applicationTerminating:note];
 }
 
-- (void)NJBTrackListModified:(NSNotification *)note
-{
-	if (!isActive)
-	{
-		trackListUpToDate = NO;
-	}
-}
-
 @end
\ No newline at end of file
